package org.hisrc.hifaces20.testing.server;

import javax.servlet.ServletContext;

/**
 * This interface models web server (like Jetty or Tomcat).
 */
public interface WebServerEnvironment {

	/**
	 * Returns the host under which this server runs.
	 * 
	 * @return Server host.
	 */
	public String getHost();

	/**
	 * Returns the port under which this server runs.
	 * 
	 * @return Server port.
	 */
	public int getPort();

	/**
	 * Returns the path of the primary context.
	 * 
	 * @return Path of the primary context.
	 */
	public String getContextPath();

	/**
	 * Returns the base URL (<code>http://server:port/</code>).
	 * 
	 * @return Base URL.
	 */
	public String getBaseUrl();

	/**
	 * Checks if the server is started.
	 * 
	 * @return <code>true</code> if the server is started, <code>false</code>
	 *         otherwise.
	 */
	public boolean isStarted();

	/**
	 * Checks if the server is stopped.
	 * 
	 * @return <code>true</code> if the server is stopped, <code>false</code>
	 *         otherwise.
	 */
	public boolean isStopped();

	/**
	 * Starts the server.
	 * 
	 * @throws Exception
	 *             In case there's a problem starting the server.
	 */
	public void start() throws Exception;

	/**
	 * Stops the server.
	 * 
	 * @throws Exception
	 *             In case there's a problem stopping.
	 */
	public void stop() throws Exception;

	/**
	 * Joins the server thread with current thread.
	 * 
	 * @throws Exception
	 *             In case there's a problem joining threads.
	 */
	public void join() throws Exception;

//	/**
//	 * Adds a web application context to this server.
//	 * 
//	 * @param contextPath
//	 *            context path (like <code>/myApp</code>).
//	 * @param location
//	 *            location - directory containing the <code>WEB-INF</code> for
//	 *            <code>myApp</code>.
//	 */
//	public void addWebAppContext(String contextPath, String location);
//
//	/**
//	 * Adds a servlet with given mapping to the web application.
//	 * 
//	 * @param contextPath
//	 *            target context path (<code>/myApp</code>).
//	 * @param servletClassName
//	 *            class name of the servler.
//	 * @param servletMapping
//	 *            servlet mapping (like <code>/myServlet</code>, the servlet
//	 *            will be accessible under <code>/myApp/myServler</code>).
//	 */
//	public void addServletWithMapping(final String contextPath,
//			final String servletClassName, final String servletMapping);

	public ServletContext getServletContext();

}
