package org.hisrc.hifaces20.testing.server.jetty;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Map.Entry;

import org.apache.commons.io.IOUtils;
import org.hisrc.hifaces20.testing.server.WebServerEnvironment;
import org.hisrc.hifaces20.testing.server.WebServerEnvironmentFactory;
import org.hisrc.hifaces20.testing.util.PropertyResolver;

public class PropertiesJettyWebServerEnvironmentFactory implements
		WebServerEnvironmentFactory {

	public static final String WEBAPP_HOST_PROPERTY_NAME = "webapp.host";
	public static final String DEFAULT_HOST = "localhost";
	public static final String WEBAPP_PORT_PROPERTY_NAME = "webapp.port";
	public static final int DEFAULT_PORT = 8080;
	public static final String WEBAPP_CONTEXT_PROPERTY_NAME = "webapp.context";
	public static final String DEFAULT_CONTEXT = "test";
	public static final String WEBAPP_HOME_PROPERTY_NAME = "webapp.home";
	public static final String DEFAULT_HOME = ".";

	private final String host;
	private final int port;
	private final String context;
	private final String home;

	public PropertiesJettyWebServerEnvironmentFactory(
			Map<String, String> properties) {

		final String host = properties.get(WEBAPP_HOST_PROPERTY_NAME);
		final String port = properties.get(WEBAPP_PORT_PROPERTY_NAME);
		final String context = properties.get(WEBAPP_CONTEXT_PROPERTY_NAME);
		final String home = properties.get(WEBAPP_HOME_PROPERTY_NAME);

		this.host = host == null ? DEFAULT_HOST : host;
		this.port = port == null ? DEFAULT_PORT : Integer.parseInt(port);
		this.context = context == null ? DEFAULT_CONTEXT : context;
		this.home = home == null ? DEFAULT_HOME : home;
	}

	public WebServerEnvironment createWebServerEnvironment() {

		final JettyWebServerEnvironment webServerEnvironment = new JettyWebServerEnvironment(
				host, port, context, home);
		return webServerEnvironment;
	}

	public static PropertiesJettyWebServerEnvironmentFactory getInstance(
			String propertiesFileName) throws IOException {
		return getInstance(new File("."), propertiesFileName);

	}

	public static PropertiesJettyWebServerEnvironmentFactory getInstance(
			File baseDir, String propertiesFileName) throws IOException {

		final Map<String, String> properties = new HashMap<String, String>();

		final File file = new File(baseDir, propertiesFileName);
		{

			final Properties draftProperties = new Properties();
			InputStream is = null;
			try {
				is = new FileInputStream(file);
				draftProperties.load(is);
			} finally {
				IOUtils.closeQuietly(is);
			}

			for (Entry<Object, Object> entry : draftProperties.entrySet()) {
				final Object key = entry.getKey();
				final Object value = entry.getValue();
				properties.put((key == null ? null : key.toString()),
						(value == null ? null : value.toString()));
			}
		}

		return getInstance(baseDir, properties);

	}

	public static PropertiesJettyWebServerEnvironmentFactory getInstance(
			final Map<String, String> properties) throws IOException {
		return getInstance(new File("."), properties);

	}

	public static PropertiesJettyWebServerEnvironmentFactory getInstance(
			File baseDir, final Map<String, String> properties)
			throws IOException {
		final Map<String, String> defaultProperties = new HashMap<String, String>();

		defaultProperties.put("baseDir", baseDir.getCanonicalFile()
				.getAbsolutePath().replace('\\', '/'));

		final Map<String, String> resolvedProperties = new PropertyResolver(
				defaultProperties).resolve(properties);

		for (Entry<String, String> entry : resolvedProperties.entrySet()) {
			System.setProperty(entry.getKey(), entry.getValue());
		}

		final PropertiesJettyWebServerEnvironmentFactory factory = new PropertiesJettyWebServerEnvironmentFactory(
				resolvedProperties);

		return factory;
	}

}
