package org.hisrc.hifaces20.testing.util;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

public class PropertyResolver {

  private final Map<String, String> properties;

  public PropertyResolver(Map<String, String> properties) {
    this.properties = (properties == null ? Collections.<String, String> emptyMap() : properties);
  }

  public Map<String, String> resolve(Map<String, String> properties) {
    if (properties == null) {
      return null;
    }
    else {

      final Map<String, String> resolvingProperties = new HashMap<String, String>(this.properties);
      resolvingProperties.putAll(properties);

      Map<String, String> currentProperties = properties;
      boolean changed = true;
      while (changed) {
        final Map<String, String> resolvedProperties = new HashMap<String, String>(currentProperties.size());
        changed = false;
        for (Entry<String, String> entry : currentProperties.entrySet()) {
          final String value = entry.getValue();
          final String resolvedValue = resolve(value, resolvingProperties);
          if (value != resolvedValue && (value == null || value.equals(resolvedValue))) {
            changed = true;
          }
          resolvedProperties.put(entry.getKey(), resolvedValue);
        }
        currentProperties = resolvedProperties;
      }
      return currentProperties;
    }
  }

  protected String resolve(String value, Map<String, String> properties) {

    if (value == null) {
      return null;
    }
    else {
      final int start = value.indexOf("${");
      final int end = value.indexOf("}");
      if (start >= 0 && end >= 0 && start < end) {
        final String name = value.substring(start + 2, end);
        final String targetValue = properties.get(name);
        if (targetValue == null) {
          return value;
        }
        else {

          final String prefix = value.substring(0, start);
          final String suffix = (end == value.length() ? "" : value.substring(end + 1));

          return resolve(prefix + targetValue + suffix, properties);
        }
      }
      else {
        return value;
      }
    }

  }

}
